@@ -1,15 +1,15 @@
 # coding=utf-8
-"""
-clock.py - Sopel Clock Module
+"""clock.py - Sopel Clock Plugin
+
 Copyright 2008-9, Sean B. Palmer, inamidst.com
 Copyright 2012, Elsie Powell, embolalia.com
 Licensed under the Eiffel Forum License 2.
 
 https://sopel.chat
 """
-from __future__ import unicode_literals, absolute_import, print_function, division
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-from sopel import module, tools
+from sopel import plugin, tools
 from sopel.tools.time import (
     format_time,
     get_channel_timezone,
@@ -18,11 +18,27 @@ from sopel.tools.time import (
     validate_timezone
 )
 
+PLUGIN_OUTPUT_PREFIX = '[clock] '
+ERROR_NO_TIMEZONE = (
+    'What timezone do you want to use? '
+    'Try one from https://sopel.chat/tz')
+ERROR_NO_FORMAT = (
+    'What format do you want me to use? '
+    'Try using http://strftime.net to make one.')
+ERROR_INVALID_TIMEZONE = (
+    'Could not find timezone "%s". '
+    'Try one from https://sopel.chat/tz')
+ERROR_INVALID_FORMAT = (
+    "That format doesn't work. Try using "
+    "http://strftime.net to make one.")
 
-@module.commands('t', 'time')
-@module.example('.t America/New_York')
-@module.example('.t Exirel')
-@module.example('.t #sopel')
+
+@plugin.command('t', 'time')
+@plugin.example('.t America/New_York', user_help=True)
+@plugin.example('.t Exirel', user_help=True)
+@plugin.example('.t #sopel', user_help=True)
+@plugin.example('.t', user_help=True)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def f_time(bot, trigger):
     """Return the current time.
 
@@ -33,11 +49,7 @@ def f_time(bot, trigger):
     it will complain. If nothing can be found, it'll complain that the argument
     is not a valid timezone.
 
-    .. seealso::
-
-        Function :func:`~sopel.tools.time.format_time` is used to format
-        the current datetime according to the timezone (if found).
-
+    The format used can be set with the ``settf`` or ``setctf`` commands.
     """
     argument = trigger.group(2)
 
@@ -59,12 +71,12 @@ def f_time(bot, trigger):
                 # zone not found for a known nick: error case
                 set_command = '%ssettz <zone>' % help_prefix
                 if channel_or_nick != trigger.nick:
-                    bot.say(
+                    bot.reply(
                         'Could not find a timezone for this nick. '
                         '%s can set a timezone with `%s`'
                         % (argument, set_command))
                 else:
-                    bot.say(
+                    bot.reply(
                         'Could not find a timezone for you. '
                         'You can set your timezone with `%s`'
                         % set_command)
@@ -74,7 +86,7 @@ def f_time(bot, trigger):
             if zone is None and channel_or_nick in bot.channels:
                 # zone not found for an existing channel: error case
                 set_command = '%ssetctz <zone>' % help_prefix
-                bot.say(
+                bot.reply(
                     'Could not find timezone for channel %s. '
                     'It can be set with `%s`. (requires OP privileges)'
                     % (argument, set_command))
@@ -86,24 +98,25 @@ def f_time(bot, trigger):
             try:
                 zone = validate_timezone(argument)
             except ValueError:
-                bot.say('Could not find timezone "%s".' % argument)
+                bot.reply(ERROR_INVALID_TIMEZONE % argument)
                 return
 
     time = format_time(bot.db, bot.config, zone, trigger.nick, trigger.sender)
     bot.say(time)
 
 
-@module.commands('tz', 'timez')
-@module.example('.tz America/New_York')
+@plugin.command('tz', 'timez')
+@plugin.example('.tz America/New_York')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def f_time_zone(bot, trigger):
     """Return the current time in a timezone.
 
-    Unlike the ``.t`` command, it requires an argument, and that argument
+    Unlike the ``time`` command, it requires an argument, and that argument
     must be a valid timezone.
     """
     argument = trigger.group(2)
     if not argument:
-        bot.say('Please provide a timezone.')
+        bot.reply(ERROR_NO_TIMEZONE)
         return
 
     zone = None
@@ -111,16 +124,15 @@ def f_time_zone(bot, trigger):
     try:
         zone = validate_timezone(argument)
     except ValueError:
-        bot.say(
-            'Cannot display time: "%s" is not a valid timezone.' % argument)
+        bot.reply(ERROR_INVALID_TIMEZONE % argument)
         return
 
     time = format_time(bot.db, bot.config, zone, trigger.nick, trigger.sender)
     bot.say(time)
 
 
-@module.commands('settz', 'settimezone')
-@module.example('.settz America/New_York')
+@plugin.command('settz', 'settimezone')
+@plugin.example('.settz America/New_York')
 def update_user(bot, trigger):
     """Set your preferred timezone.
 
@@ -129,31 +141,34 @@ def update_user(bot, trigger):
     """
     argument = trigger.group(2)
     if not argument:
-        bot.reply("What timezone do you want to set? Try one from "
-                  "https://sopel.chat/tz")
+        bot.reply(ERROR_NO_TIMEZONE)
         return
 
     try:
         zone = validate_timezone(argument)
     except ValueError:
-        bot.say(
-            'I don\'t know that timezone. Try one from https://sopel.chat/tz')
+        bot.reply(ERROR_INVALID_TIMEZONE % argument)
         return
 
     bot.db.set_nick_value(trigger.nick, 'timezone', zone)
 
     if len(zone) < 4:
-        bot.say(
-            'Okay, %s, but you should use one from https://sopel.chat/tz '
-            'if you use DST.' % trigger.nick)
+        bot.reply(
+            'Okay, but you should use one from https://sopel.chat/tz '
+            'if you use DST.')
     else:
         bot.reply('I now have you in the %s timezone.' % zone)
 
 
-@module.commands('gettz', 'gettimezone')
-@module.example('.gettz [nick]')
+@plugin.command('gettz', 'gettimezone')
+@plugin.example('.gettz Exirel', user_help=True)
+@plugin.example('.gettz', user_help=True)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def get_user_tz(bot, trigger):
-    """Gets a user's preferred time zone; will show yours if no user specified."""
+    """Gets a user's preferred time zone.
+
+    It will show yours if no user specified.
+    """
     nick = trigger.group(2)
     if not nick:
         nick = trigger.nick
@@ -167,17 +182,17 @@ def get_user_tz(bot, trigger):
         bot.say('%s has not set their time zone' % nick)
 
 
-@module.commands('settimeformat', 'settf')
-@module.example('.settf %Y-%m-%dT%T%z')
+@plugin.command('settimeformat', 'settf')
+@plugin.example('.settf %Y-%m-%dT%T%z')
 def update_user_format(bot, trigger):
-    """
-    Sets your preferred format for time. Uses the standard strftime format. You
-    can use <http://strftime.net> or your favorite search engine to learn more.
+    """Sets your preferred format for time.
+
+    Uses the standard strftime format. You can use <http://strftime.net> or
+    your favorite search engine to learn more.
     """
     tformat = trigger.group(2)
     if not tformat:
-        bot.reply("What format do you want me to use? Try using "
-                  "http://strftime.net to make one.")
+        bot.reply(ERROR_NO_FORMAT)
         return
 
     tz = get_timezone(bot.db, bot.config, None, trigger.nick, trigger.sender)
@@ -191,20 +206,26 @@ def update_user_format(bot, trigger):
     try:
         timef = format_time(db=bot.db, zone=tz, nick=trigger.nick)
     except Exception:  # TODO: Be specific
-        bot.reply("That format doesn't work. Try using "
-                  "http://strftime.net to make one.")
+        bot.reply(ERROR_INVALID_FORMAT)
         # New format doesn't work. Revert save in database.
         bot.db.set_nick_value(trigger.nick, 'time_format', old_format)
         return
-    bot.reply("Got it. Your time will now appear as %s. (If the "
-              "timezone is wrong, you might try the settz command)"
-              % timef)
+
+    set_command = '%ssettz' % bot.settings.core.help_prefix
+    bot.reply('Your time will now appear as %s. '
+              '(If the timezone is wrong, you might try the %s command)'
+              % (timef, set_command))
 
 
-@module.commands('gettimeformat', 'gettf')
-@module.example('.gettf [nick]')
+@plugin.command('gettimeformat', 'gettf')
+@plugin.example('.gettf Exirel', user_help=True)
+@plugin.example('.gettf', user_help=True)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def get_user_format(bot, trigger):
-    """Gets a user's preferred time format; will show yours if no user specified."""
+    """Gets a user's preferred time format.
+
+    It will show yours if no user specified.
+    """
     nick = trigger.group(2)
     if not nick:
         nick = trigger.nick
@@ -220,41 +241,42 @@ def get_user_format(bot, trigger):
         bot.say("%s hasn't set a custom time format" % nick)
 
 
-@module.commands('setchanneltz', 'setctz')
-@module.example('.setctz America/New_York')
-@module.require_privilege(module.OP, message='Changing the channel timezone requires OP privileges.')
+@plugin.command('setchanneltz', 'setctz')
+@plugin.example('.setctz America/New_York')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, message='Changing the channel timezone requires OP privileges.')
 def update_channel(bot, trigger):
-    """Set the preferred timezone for the channel."""
+    """Set the preferred timezone for the current channel."""
     argument = trigger.group(2)
     if not argument:
-        bot.reply("What timezone do you want to set? Try one from "
-                  "https://sopel.chat/tz")
+        bot.reply(ERROR_NO_TIMEZONE)
         return
 
     try:
         zone = validate_timezone(argument)
     except ValueError:
-        bot.say(
-            'I don\'t know that timezone. Try one from https://sopel.chat/tz')
+        bot.reply(ERROR_INVALID_TIMEZONE % argument)
         return
 
     channel = trigger.sender
     bot.db.set_channel_value(channel, 'timezone', zone)
 
     if len(zone) < 4:
-        bot.say(
-            'Okay, %s, but you should use one from https://sopel.chat/tz '
-            'if you use DST.' % trigger.nick)
+        bot.reply(
+            'Okay, but you should use one from https://sopel.chat/tz '
+            'if you use DST.')
     else:
         bot.reply('I now have %s in the %s timezone.' % (channel, zone))
 
 
-@module.commands('getchanneltz', 'getctz')
-@module.example('.getctz [channel]')
+@plugin.command('getchanneltz', 'getctz')
+@plugin.example('.getctz #sopel', user_help=True)
+@plugin.example('.getctz', user_help=True)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def get_channel_tz(bot, trigger):
-    """
-    Gets the channel's preferred timezone; returns the current channel's
-    if no channel name is given.
+    """Gets the channel's preferred timezone.
+
+    It returns the current channel's if no channel name is given.
     """
     channel = trigger.group(2)
     if not channel:
@@ -269,18 +291,21 @@ def get_channel_tz(bot, trigger):
         bot.say('%s has no preferred timezone' % channel)
 
 
-@module.commands('setchanneltimeformat', 'setctf')
-@module.example('.setctf %Y-%m-%dT%T%z')
-@module.require_privilege(module.OP)
+@plugin.command('setchanneltimeformat', 'setctf')
+@plugin.example('.setctf %Y-%m-%dT%T%z')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def update_channel_format(bot, trigger):
-    """
-    Sets your preferred format for time. Uses the standard strftime format. You
-    can use <http://strftime.net> or your favorite search engine to learn more.
+    """Sets your preferred format for time.
+
+    Uses the standard strftime format. You can use <http://strftime.net> or
+    your favorite search engine to learn more.
     """
     tformat = trigger.group(2)
     if not tformat:
-        bot.reply("What format do you want me to use? Try using "
-                  "http://strftime.net to make one.")
+        bot.reply(ERROR_NO_FORMAT)
+        return
 
     tz = get_timezone(bot.db, bot.config, None, None, trigger.sender)
 
@@ -293,26 +318,30 @@ def update_channel_format(bot, trigger):
     try:
         timef = format_time(db=bot.db, zone=tz, channel=trigger.sender)
     except Exception:  # TODO: Be specific
-        bot.reply("That format doesn't work. Try using"
-                  " http://strftime.net to make one.")
+        bot.reply(ERROR_INVALID_FORMAT)
         # New format doesn't work. Revert save in database.
         bot.db.set_channel_value(trigger.sender, 'time_format', old_format)
         return
     bot.db.set_channel_value(trigger.sender, 'time_format', tformat)
-    bot.reply("Got it. Times in this channel  will now appear as %s "
-              "unless a user has their own format set. (If the timezone"
-              " is wrong, you might try the settz and channeltz "
-              "commands)" % timef)
+
+    help_prefix = bot.settings.core.help_prefix
+    set_command = '%ssettz' % help_prefix
+    channel_command = '%schanneltz' % help_prefix
+    bot.say("Times in this channel will now appear as %s "
+            "unless a user has their own format set. (If the timezone"
+            " is wrong, you might try the %s and %s "
+            "commands)" % (timef, set_command, channel_command))
 
 
-@module.commands('getchanneltimeformat', 'getctf')
-@module.example('.getctf [channel]')
+@plugin.command('getchanneltimeformat', 'getctf')
+@plugin.example('.getctf #sopel', user_help=True)
+@plugin.example('.getctf', user_help=True)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def get_channel_format(bot, trigger):
-    """
-    Gets the channel's preferred time format; will return current channel's if
-    no channel name is given.
-    """
+    """Gets the channel's preferred time format
 
+    It returns the current channel's if no channel name is given.
+    """
     channel = trigger.group(2)
     if not channel:
         channel = trigger.sender
